/**
 * @author Yi-Qing Wang <yiqing.wang@cmla.ens-cachan.fr>
 * All rights reserved.
 */

#include "global.hpp"

#include <stdlib.h>

#include "build.hpp"

#include "NeuralStruct.hpp"

/* call info */
static int _argc;
static const char * _arg0;

static void usage() {
	fprintf(stderr, \
	        "syntax:\n"
	        "	%s -h\n"
	        "	%s -c clean [-m noisy] [-d denoised]\n"
	        "options:\n"
	        "    	-h print this help\n"
	        "	-c path to a clean input image\n"
	        "	-m path to store the noisy image (default ./noisy.png)\n"
	        "	-d path to store the NN denoised image (default ./nndenoised.png)\n",
	        _arg0, _arg0);
	exit(EXIT_SUCCESS);
}

/**
 * increment argument index without going too far
 */
static int shift(int &i) {
	i++;
	if (i >= _argc)
		ERROR("missing parameter value");
	return i;
}

int main(int argc, char ** argv) {

	const char * clean = "unspecified";
	const char * denoised = "./nndenoised.png";
	const char * noisy = "./noisy.png";

	// set global call info
	_argc = argc;
	_arg0 = argv[0];

	for (int i = 1; i < argc; i++) {
		if (0 == strcmp(argv[i],"-h")) {
			usage();
		} else if (0 == strcmp(argv[i],"-c")) {
			clean = argv[shift(i)];
		} else if (0 == strcmp(argv[i],"-d")) {
			denoised = argv[shift(i)];
		} else if (0 == strcmp(argv[i],"-m")) {
			noisy = argv[shift(i)];
		} else
			ERROR("unknown parameter");

	}

	if (0 == strcmp(clean, "unspecified"))
		ERROR("Unspecified input (print the manual with %s -h)", _arg0);

	denoise(clean, denoised, noisy);
}
